'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _defineProperty2 = require('babel-runtime/helpers/defineProperty');

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _defineProperty4 = require('babel-runtime/core-js/object/define-property');

var _defineProperty5 = _interopRequireDefault(_defineProperty4);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _pipeline = require('../pipeline');

var pipeline = _interopRequireWildcard(_pipeline);

var _graphCrdt = require('graph-crdt');

var _root = require('./root');

var _root2 = _interopRequireDefault(_root);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Asynchronous node interface.
 * @class
 */
var Context = function (_Node) {
  (0, _inherits3.default)(Context, _Node);

  /**
   * Creates a new context.
   * @param {Object} root - The root database instance.
   * @param {Object} [node] - Node configurations.
   */
  function Context(root, node) {
    (0, _classCallCheck3.default)(this, Context);

    var _this = (0, _possibleConstructorReturn3.default)(this, (Context.__proto__ || (0, _getPrototypeOf2.default)(Context)).call(this, node));

    _this.root = root;

    if (root) {
      var config = root[_root2.default.configuration];
      var extend = config.extend.context;
      (0, _keys2.default)(extend).forEach(function (key) {
        var value = extend[key];

        (0, _defineProperty5.default)(_this, key, { value: value });
      });
    }
    return _this;
  }

  /**
   * Reads a value from the node.
   * @param  {String} field - The field to read.
   * @param  {String} [options] - Plugin-level options.
   * @return {Promise} - Resolves to the value or undefined.
   */


  (0, _createClass3.default)(Context, [{
    key: 'read',
    value: function () {
      var _ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee(field, options) {
        var config, params, result, _ref2, value;

        return _regenerator2.default.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                config = this.root[_root2.default.configuration];
                _context.next = 3;
                return pipeline.before.read.field(config, (0, _extends3.default)({}, options, {
                  node: this,
                  field: field
                }));

              case 3:
                params = _context.sent;


                /** Read from the node. */
                result = this.value(params.field);

                /** Detect and resolve pointers. */

                if (!(result instanceof Object)) {
                  _context.next = 9;
                  break;
                }

                _context.next = 8;
                return this.root.read(result.edge, params);

              case 8:
                result = _context.sent;

              case 9:
                _context.next = 11;
                return pipeline.after.read.field(config, (0, _extends3.default)({}, params, {
                  value: result
                }));

              case 11:
                _ref2 = _context.sent;
                value = _ref2.value;
                return _context.abrupt('return', value);

              case 14:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      function read(_x, _x2) {
        return _ref.apply(this, arguments);
      }

      return read;
    }()

    /**
     * Writes a value to the node.
     * @param  {String} field - Where to write the data.
     * @param  {Mixed} value - A value to write.
     * @return {Promise} - Resolves when the value has written successfully.
     */

  }, {
    key: 'write',
    value: function () {
      var _ref3 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee2(field, value) {
        var delta, update;
        return _regenerator2.default.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:

                /** It's a reference. Save as a pointer. */
                if (value instanceof _graphCrdt.Node) {
                  value = { edge: String(value) };
                }

                /** Create a node delta. */
                delta = this.new();

                /** Bump the state if it exists. */

                if (this.state(field)) {
                  delta.merge((0, _defineProperty3.default)({}, field, this.value(field)));
                  delta.meta(field).state = this.state(field);
                }

                /** Apply the field update. */
                delta.merge((0, _defineProperty3.default)({}, field, value));

                /** Create a graph delta. */
                update = _graphCrdt.Graph.source((0, _defineProperty3.default)({}, this.meta().uid, delta));

                /** Commit the update. */

                _context2.next = 7;
                return this.root.commit(update);

              case 7:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));

      function write(_x3, _x4) {
        return _ref3.apply(this, arguments);
      }

      return write;
    }()

    /**
     * Creates a copy of the context.
     * @return {Context} - A new context with the same uid and root.
     */

  }, {
    key: 'new',
    value: function _new() {
      var config = this.meta();

      return new Context(this.root, config);
    }
  }]);
  return Context;
}(_graphCrdt.Node);

exports.default = Context;