'use strict';

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _defineProperty2 = require('babel-runtime/helpers/defineProperty');

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

var _expect = require('expect');

var _expect2 = _interopRequireDefault(_expect);

var _root = require('../root');

var _root2 = _interopRequireDefault(_root);

var _context6 = require('../context');

var _context7 = _interopRequireDefault(_context6);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

describe('A context', function () {
  var node = void 0,
      root = void 0;

  beforeEach(function () {
    root = (0, _root2.default)();

    var context = new _context7.default(root);

    var _context$meta = context.meta(),
        uid = _context$meta.uid;

    root.merge((0, _defineProperty3.default)({}, uid, context));
    node = root.value(uid);
  });

  it('should expose the database root', function () {
    (0, _expect2.default)(node.root).toBe(root);
  });

  it('should accept node constructor settings', function () {
    node = new _context7.default(root, {
      uid: 'potatoes'
    });

    var _node$meta = node.meta(),
        uid = _node$meta.uid;

    (0, _expect2.default)(uid).toBe('potatoes');
  });

  describe('read', function () {

    it('should return undefined if not found', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee() {
      var result;
      return _regenerator2.default.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              _context.next = 2;
              return node.read('nothing here');

            case 2:
              result = _context.sent;

              (0, _expect2.default)(result).toBe(undefined);

            case 4:
            case 'end':
              return _context.stop();
          }
        }
      }, _callee, undefined);
    })));

    it('should return the value if it exists', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee2() {
      var enabled;
      return _regenerator2.default.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              _context2.next = 2;
              return node.write('enabled', true);

            case 2:
              _context2.next = 4;
              return node.read('enabled');

            case 4:
              enabled = _context2.sent;


              (0, _expect2.default)(enabled).toBe(true);

            case 6:
            case 'end':
              return _context2.stop();
          }
        }
      }, _callee2, undefined);
    })));

    it('should resolve contexts through the root', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee3() {
      var settings, read, result;
      return _regenerator2.default.wrap(function _callee3$(_context3) {
        while (1) {
          switch (_context3.prev = _context3.next) {
            case 0:
              settings = new _context7.default(root, { uid: 'settings' });
              _context3.next = 3;
              return node.write('settings', settings);

            case 3:
              read = (0, _expect.spyOn)(root, 'read');

              read.andReturn(_promise2.default.resolve(settings));

              _context3.next = 7;
              return node.read('settings');

            case 7:
              result = _context3.sent;

              (0, _expect2.default)(result).toBe(settings);

              read.restore();

            case 10:
            case 'end':
              return _context3.stop();
          }
        }
      }, _callee3, undefined);
    })));
  });

  describe('write', function () {

    it('should write as an edge when given a context', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee4() {
      var settings, _node$meta2, value;

      return _regenerator2.default.wrap(function _callee4$(_context4) {
        while (1) {
          switch (_context4.prev = _context4.next) {
            case 0:
              settings = new _context7.default(root, { uid: 'settings' });
              _context4.next = 3;
              return node.write('settings', settings);

            case 3:
              _node$meta2 = node.meta('settings'), value = _node$meta2.value;

              (0, _expect2.default)(value).toEqual({ edge: 'settings' });

            case 5:
            case 'end':
              return _context4.stop();
          }
        }
      }, _callee4, undefined);
    })));

    it('increments node state', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee5() {
      return _regenerator2.default.wrap(function _callee5$(_context5) {
        while (1) {
          switch (_context5.prev = _context5.next) {
            case 0:
              _context5.next = 2;
              return node.write('count', 5);

            case 2:
              (0, _expect2.default)(node.value('count')).toBe(5);

              _context5.next = 5;
              return node.write('count', 7);

            case 5:
              (0, _expect2.default)(node.value('count')).toBe(7);

              _context5.next = 8;
              return node.write('count', 3);

            case 8:
              (0, _expect2.default)(node.value('count')).toBe(3);

              (0, _expect2.default)(node.state('count')).toBe(3);

            case 10:
            case 'end':
              return _context5.stop();
          }
        }
      }, _callee5, undefined);
    })));
  });

  describe('"new" call', function () {

    it('should return a new context', function () {
      var copy = node.new();
      (0, _expect2.default)(copy).toBeA(_context7.default);
    });

    it('should use the same uid', function () {
      var copy = node.new();
      (0, _expect2.default)(copy.meta()).toContain({
        uid: String(node)
      });
    });

    it('should attach the same root reference', function () {
      var copy = node.new();
      (0, _expect2.default)(copy.root).toBe(root);
    });

    it('should include API extensions', function () {
      var root = (0, _root2.default)({
        extend: {
          context: { hello: 'coder' }
        }
      });

      var ctx = new _context7.default(root);
      var copy = ctx.new();
      (0, _expect2.default)(copy.hello).toBe('coder');
    });
  });

  describe('API extension', function () {
    var extend = function extend(context) {
      return (0, _root2.default)({
        extend: { context: context }
      });
    };

    it('should contain extensions from the config', function () {
      var root = extend({
        wazzup: function wazzup() {}
      });

      var ctx = new _context7.default(root);
      (0, _expect2.default)(ctx.wazzup).toBeA(Function);
    });

    it('should be non-enumerable', function () {
      var root = extend({ 'non-enumerable': true });

      var ctx = new _context7.default(root);
      var keys = (0, _keys2.default)(ctx);
      (0, _expect2.default)(keys).toNotContain('non-enumerable');
    });

    it('should be immutable', function () {
      var root = extend({ hey: 'there' });

      var ctx = new _context7.default(root);
      (0, _expect2.default)(function () {
        ctx.hey = 'change!';
      }).toThrow(Error);
    });
  });
}); /* eslint-env mocha */