'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.after = exports.before = exports.defaults = undefined;

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _connectionGroup = require('../connection-group');

var _connectionGroup2 = _interopRequireDefault(_connectionGroup);

var _triggerHooks = require('../trigger-hooks');

var _triggerHooks2 = _interopRequireDefault(_triggerHooks);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Sets default options for methods like `.read` & `.write`.
 * @param  {Object} config - The database configuration object.
 * @param  {Object} [options] - Options passed to read/write.
 * @return {Object} - An options object with defaults set via the config.
 */
var defaults = exports.defaults = function defaults(config) {
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

  var provided = {
    storage: options.storage !== undefined,
    network: options.network !== undefined
  };

  var storage = provided.storage ? options.storage : config.storage;
  var network = provided.network ? options.network : config.network;

  return (0, _extends3.default)({}, options, {

    storage: storage || [],
    network: network || new _connectionGroup2.default()
  });
};

/**
 * Returns whatever it's passed.
 * @param  {Mixed} value - Any value.
 * @return {Mixed} - That same value.
 */
var identity = function identity(value) {
  return value;
};

/**
 * Creates an event pipeline.
 * @param  {String} path - The path to the hooks inside a hooks object.
 * @param  {Function} [transform] - Handles special return values from hooks.
 * @return {Function} - Triggers a pipeline.
 */
var createPipeline = function createPipeline(path) {
  var transform = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : identity;
  return (

    /**
     * Triggers a pipeline.
     * @param  {Object} config - Database configuration object.
     * @param  {Array} options - The initial parameters.
     * @return {Promise} - Resolves to the pipeline output.
     */
    function (config, options) {
      return (0, _triggerHooks2.default)({
        hooks: path.reduce(function (hooks, type) {
          return hooks[type];
        }, config.hooks),
        initial: defaults(config, options),
        transform: transform
      });
    }
  );
};

var before = exports.before = {
  read: {
    node: createPipeline(['before', 'read', 'node']),
    field: createPipeline(['before', 'read', 'field'])
  },

  write: createPipeline(['before', 'write']),
  request: createPipeline(['before', 'request']),
  update: createPipeline(['before', 'update'])
};

var after = exports.after = {
  read: {
    node: createPipeline(['after', 'read', 'node']),
    field: createPipeline(['after', 'read', 'field'])
  },

  write: createPipeline(['after', 'write']),
  request: createPipeline(['after', 'request']),
  update: createPipeline(['after', 'update'])
};